@@ -18,8 +18,6 @@
 #
 package Output;
 use strict;
-use open IO => ":utf8";
-
 BEGIN { @Namespace::ISA=qw(main); }
 
 # $objOutputter new()
@@ -32,7 +30,7 @@ sub new
   $$self{out} = [];
 
   $$self{source} = "";
-  $$self{tmpdir} = File::Spec->tmpdir();
+  $$self{tmpdir} = "/tmp";
   $$self{destdir} = "";
   $$self{objDefsParser} = undef; # It will be set in set_defsparser()
 
@@ -80,33 +78,15 @@ sub error
   printf STDERR "Output.pm: $format",@_;
 }
 
-sub ifdef($$)
-{
-	my ($self, $ifdef) = @_;
-	if ($ifdef)
-	{
-		$self->append("\n#ifdef $ifdef\n");
-	}
-}
-
-sub endif($$)
-{
-	my ($self, $ifdef) = @_;
-	if ($ifdef)
-	{
-		$self->append("\n#endif // $ifdef\n");
-	}
-}
-
 ### Convert _WRAP to a virtual 
 # _VFUNC_H(signame,rettype,`<cppargs>')
 # _VFUNC_PH(gtkname,crettype,cargs and names)
 # void output_wrap_vfunc_h($filename, $line_num, $objCppfunc, $objCDefsFunc)
-sub output_wrap_vfunc_h($$$$$$)
+sub output_wrap_vfunc_h($$$$$)
 {
-  my ($self, $filename, $line_num, $objCppfunc, $objCDefsFunc, $ifdef) = @_;
+  my ($self, $filename, $line_num, $objCppfunc, $objCDefsFunc) = @_;
 
-#Old code. We removed _VFUNC_H from the .m4 file
+#TODO: We can probably remove _VFUNC_H from the .m4 file
 #  my $str = sprintf("_VFUNC_H(%s,%s,\`%s\',%s)dnl\n",
 #    $$objCppfunc{name},
 #    $$objCppfunc{rettype},
@@ -121,89 +101,76 @@ sub output_wrap_vfunc_h($$$$$$)
     $cppVfuncDecl .= " const";
   }
 
-  $self->ifdef($ifdef);
-  $self->append("  $cppVfuncDecl;\n");
-  $self->endif($ifdef);
+  $self->append("  $cppVfuncDecl;");
 
   #The default callback, which will call *_vfunc, which will then call the base default callback.
   #Declares the callback in the private *Class class and sets it in the class_init function.
 
-  my $str = sprintf("_VFUNC_PH(%s,%s,\`%s\',%s)dnl\n",
+  my $str = sprintf("_VFUNC_PH(%s,%s,\`%s\')dnl\n",
     $$objCDefsFunc{name},
     $$objCDefsFunc{rettype},
     $objCDefsFunc->args_types_and_names(),
-    $ifdef
    );
   $self->append($str);
 }
 
 # _VFUNC_CC(signame,gtkname,rettype,crettype,`<cppargs>',`<cargs>')
-sub output_wrap_vfunc_cc($$$$$$$)
+sub output_wrap_vfunc_cc($$$$$$)
 {
-  my ($self, $filename, $line_num, $objCppfunc, $objCFunc,
-      $custom_vfunc, $custom_vfunc_callback, $ifdef) = @_;
+  my ($self, $filename, $line_num, $objCppfunc, $objDefsSignal) = @_;
 
-  my $cname = $$objCFunc{name};
+  my $cname = $$objDefsSignal{name};
 
   # e.g. Gtk::Button::draw_indicator:
 
   #Use a different macro for Interfaces, to generate an extra convenience method.
 
-  if ($custom_vfunc eq 0)
-  {
-    my $refreturn = "";
-    $refreturn = "refreturn" if($$objCppfunc{rettype_needs_ref});
+  my $refreturn = "";
+  $refreturn = "refreturn" if($$objCppfunc{rettype_needs_ref});
 
-    my $str = sprintf("_VFUNC_CC(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s,%s)dnl\n",
-      $$objCppfunc{name},
-      $cname,
-      $$objCppfunc{rettype},
-      $$objCFunc{rettype},
-      $objCppfunc->args_types_and_names(),
-      convert_args_cpp_to_c($objCppfunc, $objCFunc, 0, $line_num), #$objCppfunc->args_names_only(),
-      $objCppfunc->get_is_const(),
-      $refreturn,
-      $ifdef);
+  my $str = sprintf("_VFUNC_CC(%s,%s,%s,%s,\`%s\',\`%s\',%s)dnl\n",
+    $$objCppfunc{name},
+    $cname,
+    $$objCppfunc{rettype},
+    $$objDefsSignal{rettype},
+    $objCppfunc->args_types_and_names(),
+    convert_args_cpp_to_c($objCppfunc, $objDefsSignal, 0, $line_num), #$objCppfunc->args_names_only(),
+    $objCppfunc->get_is_const(),
+    $refreturn);
 
-    $self->append($str);
-  }
+  $self->append($str);
 
   # e.g. Gtk::ButtonClass::draw_indicator():
 
-  if ($custom_vfunc_callback eq 0)
-  {
-    my $refreturn_ctype = "";
-    $refreturn_ctype = "refreturn_ctype" if($$objCFunc{rettype_needs_ref});
+  my $refreturn_ctype = "";
+  $refreturn_ctype = "refreturn_ctype" if($$objDefsSignal{rettype_needs_ref});
 
-    my $str = sprintf("_VFUNC_PCC(%s,%s,%s,%s,\`%s\',\`%s\',\`%s\',%s,%s,%s)dnl\n",
-      $$objCppfunc{name},
-      $cname,
-      $$objCppfunc{rettype},
-      $$objCFunc{rettype},
-      $objCFunc->args_types_and_names(),
-      $objCFunc->args_names_only(),
-      convert_args_c_to_cpp($objCFunc, $objCppfunc, $line_num),
-      ${$objCFunc->get_param_names()}[0],
-      $refreturn_ctype,
-      $ifdef);
+  my $str = sprintf("_VFUNC_PCC(%s,%s,%s,%s,\`%s\',\`%s\',\`%s\',%s,%s)dnl\n",
+    $$objCppfunc{name},
+    $cname,
+    $$objCppfunc{rettype},
+    $$objDefsSignal{rettype},
+    $objDefsSignal->args_types_and_names(),
+    $objDefsSignal->args_names_only(),
+    convert_args_c_to_cpp($objDefsSignal, $objCppfunc, $line_num),
+    ${$objDefsSignal->get_param_names()}[0],
+    $refreturn_ctype);
 
-    $self->append($str);
-  }
+  $self->append($str);
 }
 
-### Convert _WRAP to a signal
-# _SIGNAL_H(signame,rettype, ifdef, `<cppargs>')
-# _SIGNAL_PH(gtkname,crettype, ifdef, cargs and names)
-# void output_wrap_default_signal_handler_h($filename, $line_num, $objCppfunc, $objCDefsFunc, $ifdef. @args)
-sub output_wrap_default_signal_handler_h($$$$$$$)
+### Convert _WRAP to a virtual
+# _SIGNAL_H(signame,rettype,`<cppargs>')
+# _SIGNAL_PH(gtkname,crettype,cargs and names)
+# void output_wrap_default_signal_handler_h($filename, $line_num, $objCppfunc, $objCDefsFunc, @args)
+sub output_wrap_default_signal_handler_h($$$$$$)
 {
-  my ($self, $filename, $line_num, $objCppfunc, $objCDefsFunc, $ifdef) = @_;
+  my ($self, $filename, $line_num, $objCppfunc, $objCDefsFunc, $bImplement) = @_;
 
-  my $str = sprintf("_SIGNAL_H(%s,%s,\`%s\',%s)dnl\n",
+  my $str = sprintf("_SIGNAL_H(%s,%s,\`%s\')dnl\n",
     $$objCppfunc{name},
     $$objCppfunc{rettype},
-    $objCppfunc->args_types_and_names(),
-    $ifdef
+    $objCppfunc->args_types_and_names()
    );
   $self->append($str);
 
@@ -211,19 +178,18 @@ sub output_wrap_default_signal_handler_h($$$$$$$)
   #The default callback, which will call *_impl, which will then call the base default callback.
   #Declares the callback in the private *Class class and sets it in the class_init function.
 
-  $str = sprintf("_SIGNAL_PH(%s,%s,\`%s\',%s)dnl\n",
+  $str = sprintf("_SIGNAL_PH(%s,%s,\`%s\')dnl\n",
     $$objCDefsFunc{name},
     $$objCDefsFunc{rettype},
-    $objCDefsFunc->args_types_and_names(),
-    $ifdef
+    $objCDefsFunc->args_types_and_names()
    );
   $self->append($str);
 }
 
 # _SIGNAL_CC(signame, gtkname, rettype, crettype,`<cppargs>',`<cargs>')
-sub output_wrap_default_signal_handler_cc($$$$$$$$$)
+sub output_wrap_default_signal_handler_cc($$$$$$$$)
 {
-  my ($self, $filename, $line_num, $objCppfunc, $objDefsSignal, $bImplement, $bCustomCCallback, $bRefreturn, $ifdef) = @_;
+  my ($self, $filename, $line_num, $objCppfunc, $objDefsSignal, $bImplement, $bCustomCCallback, $bRefreturn) = @_;
   my $cname = $$objDefsSignal{name};
   # $cname = $1 if ($args[3] =~ /"(.*)"/); #TODO: What's this about?
 
@@ -233,7 +199,7 @@ sub output_wrap_default_signal_handler_cc($$$$$$$$$)
     my $refreturn = "";
     $refreturn = "refreturn" if($bRefreturn eq 1);
   
-    my $str = sprintf("_SIGNAL_CC(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s,%s)dnl\n",
+    my $str = sprintf("_SIGNAL_CC(%s,%s,%s,%s,\`%s\',\`%s\',%s, %s)dnl\n",
       $$objCppfunc{name},
       $cname,
       $$objCppfunc{rettype},
@@ -241,8 +207,7 @@ sub output_wrap_default_signal_handler_cc($$$$$$$$$)
       $objCppfunc->args_types_and_names(),
       convert_args_cpp_to_c($objCppfunc, $objDefsSignal, 0, $line_num), #$objCppfunc->args_names_only(),
       $$objCppfunc{const},
-      $refreturn,
-      $ifdef);
+      $refreturn);
     $self->append($str);
   }
 
@@ -260,7 +225,7 @@ sub output_wrap_default_signal_handler_cc($$$$$$$$$)
 
   if($bCustomCCallback ne 1)
   {
-    my $str = sprintf("_SIGNAL_PCC(%s,%s,%s,%s,\`%s\',\`%s\',\`%s\',\`%s\',%s)dnl\n",
+    my $str = sprintf("_SIGNAL_PCC(%s,%s,%s,%s,\`%s\',\`%s\',\`%s\',%s)dnl\n",
       $$objCppfunc{name},
       $cname,
       $$objCppfunc{rettype},
@@ -268,70 +233,40 @@ sub output_wrap_default_signal_handler_cc($$$$$$$$$)
       $objDefsSignal->args_types_and_names(),
       $objDefsSignal->args_names_only(),
       convert_args_c_to_cpp($objDefsSignal, $objCppfunc, $line_num),
-      ${$objDefsSignal->get_param_names()}[0],
-      $ifdef);
+      ${$objDefsSignal->get_param_names()}[0]);
     $self->append($str);
   }
 }
 
 ### Convert _WRAP to a method
 #  _METHOD(cppname,cname,cpprettype,crettype,arglist,cargs,const)
-#  void output_wrap_meth($filename, $line_num, $objCppFunc, $objCDefsFunc, $cppMethodDecl, $documentation, $ifdef)
-sub output_wrap_meth($$$$$$$)
+#  void output_wrap_meth($filename, $line_num, $objCppFunc, $objCDefsFunc, $cppMethodDecl, $documentation)
+sub output_wrap_meth($$$$$$)
 {
-  my ($self, $filename, $line_num, $objCppfunc, $objCDefsFunc, $cppMethodDecl, $documentation, $ifdef) = @_;
+  my ($self, $filename, $line_num, $objCppfunc, $objCDefsFunc, $cppMethodDecl, $documentation) = @_;
   my $objDefsParser = $$self{objDefsParser};
 
-  # Allow the generated .h/.cc code to have an #ifndef around it, and add deprecation docs to the generated documentation.
-  my $deprecated = "";
-  if($$objCDefsFunc{deprecated})
-  {
-    $deprecated = "deprecated";
-  }
-
-  #Declaration:
-  if($deprecated ne "")
-  {
-    $self->append("\n_DEPRECATE_IFDEF_START");
-  }
-
   # Doxygen documentation before the method declaration:
   $self->output_wrap_meth_docs_only($filename, $line_num, $documentation);
 
-  $self->ifdef($ifdef);
-
+  #Declaration:
   $self->append("  ${cppMethodDecl};");
-  
-  $self->endif($ifdef);
-
-  
-  if($deprecated ne "")
-  {
-    $self->append("\n_DEPRECATE_IFDEF_END\n");
-  }
 
   my $refneeded = "";
   if($$objCDefsFunc{rettype_needs_ref})
   {
     $refneeded = "refreturn"
   }
-
   my $errthrow = "";
   if($$objCDefsFunc{throw_any_errors})
   {
     $errthrow = "errthrow"
   }
 
-  my $constversion = ""; #Whether it is just a const overload (so it can reuse code)
-  if($$objCDefsFunc{constversion})
-  {
-    $constversion = "constversion"
-  }
-
   #Implementation:
   my $str;
   if ($$objCppfunc{static}) {
-    $str = sprintf("_STATIC_METHOD(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s,%s,%s)dnl\n",
+    $str = sprintf("_STATIC_METHOD(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s)dnl\n",
       $$objCppfunc{name},
       $$objCDefsFunc{c_name},
       $$objCppfunc{rettype},
@@ -339,11 +274,9 @@ sub output_wrap_meth($$$$$$$)
       $objCppfunc->args_types_and_names(),
       convert_args_cpp_to_c($objCppfunc, $objCDefsFunc, 1, $line_num, $errthrow), #1 means it's static, so it has 'object'.
       $refneeded,
-      $errthrow,
-      $deprecated,
-      $ifdef);
+      $errthrow);
   } else {
-    $str = sprintf("_METHOD(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s,%s,%s,%s,\`%s\',%s)dnl\n",
+    $str = sprintf("_METHOD(%s,%s,%s,%s,\`%s\',\`%s\',%s,%s,%s)dnl\n",
       $$objCppfunc{name},
       $$objCDefsFunc{c_name},
       $$objCppfunc{rettype},
@@ -352,13 +285,10 @@ sub output_wrap_meth($$$$$$$)
       convert_args_cpp_to_c($objCppfunc, $objCDefsFunc, 0, $line_num, $errthrow),
       $$objCppfunc{const},
       $refneeded,
-      $errthrow,
-      $deprecated,
-      $constversion,
-      $objCppfunc->args_names_only(),
-      $ifdef
-      );
+      $errthrow);
   }
+
+
   $self->append($str);
 }
 
@@ -417,41 +347,15 @@ sub output_wrap_create($$$)
 
 # void output_wrap_sig_decl($filename, $line_num, $objCSignal, $objCppfunc, $signal_name, $bCustomCCallback)
 # custom_signalproxy_name is "" when no type conversion is required - a normal templates SignalProxy will be used instead.
-sub output_wrap_sig_decl($$$$$$$$)
+sub output_wrap_sig_decl($$$$$$$)
 {
-  my ($self, $filename, $line_num, $objCSignal, $objCppfunc, $signal_name, $bCustomCCallback, $ifdef, $merge_doxycomment_with_previous) = @_;
+  my ($self, $filename, $line_num, $objCSignal, $objCppfunc, $signal_name, $bCustomCCallback) = @_;
 
 # _SIGNAL_PROXY(c_signal_name, c_return_type, `<c_arg_types_and_names>',
 #               cpp_signal_name, cpp_return_type, `<cpp_arg_types>',`<c_args_to_cpp>',
 #               refdoc_comment)
 
-  my $doxycomment = $objCppfunc->get_refdoc_comment();
-
-  # If there was already a previous doxygen comment, we want to merge this
-  # one with the previous so it is one big comment. If it were two separate
-  # comments, doxygen would ignore the first one. If
-  # $merge_doxycomment_with_previous is nonzero, the first comment is
-  # already open but not yet closed.
-  if($merge_doxycomment_with_previous)
-  {
-    # Strip leading whitespace
-    $doxycomment =~ s/^\s+//;
-    # We don't have something to add, so just close the comment.
-    if($doxycomment eq "")
-    {
-      $doxycomment = "   */";
-    }
-    else
-    {
-      # Append the new comment, but remove the first three leading characters
-      # (which are /**) that mark the beginning of the comment.
-      $doxycomment = substr($doxycomment, 3);
-      $doxycomment =~ s/^\s+//;
-      $doxycomment = "   " . $doxycomment;
-    }
-  }
-
-  my $str = sprintf("_SIGNAL_PROXY(%s,%s,\`%s\',%s,%s,\`%s\',\`%s\',\`%s\',\`%s\',%s)dnl\n",
+  my $str = sprintf("_SIGNAL_PROXY(%s,%s,\`%s\',%s,%s,\`%s\',\`%s\',\`%s\')dnl\n",
     $signal_name,
     $$objCSignal{rettype},
     $objCSignal->args_types_and_names_without_object(),
@@ -460,8 +364,7 @@ sub output_wrap_sig_decl($$$$$$$$)
     $objCppfunc->args_types_only(),
     convert_args_c_to_cpp($objCSignal, $objCppfunc, $line_num),
     $bCustomCCallback, #When this is true, it will not write the *_callback implementation for you.
-    $doxycomment,
-    $ifdef
+    $objCppfunc->get_refdoc_comment()
   );
 
   $self->append($str);
@@ -568,12 +471,6 @@ sub output_wrap_property($$$$$$)
     {
       # construct-only functions can be read, but not written.
       $proxy_suffix = "_ReadOnly";
-
-      if($objProperty->get_readable() ne 1)
-      {
-        $self->output_wrap_failed($name, "attempt to wrap write-only and construct-only property.");
-        return;
-      }
     }
     elsif($objProperty->get_readable() ne 1)
     {
@@ -586,16 +483,9 @@ sub output_wrap_property($$$$$$)
 
     # Convert - to _ so we can use it in C++ method and variable names:
     my $name_underscored = $name;
-    $name_underscored =~ tr/-/_/;
+    $name_underscored =~ s/-/_/g;
 
-      # For the docs of the property (the final argument of the sprintf), the
-      # m4 quotes are changed, the new quotes are then used to quote the docs
-      # and then the quotes are changed back to the standard quotes.  This is
-      # done so that if there are commas in the docs, the contents after the
-      # docs are not lost (m4 thinks the contents after the comma is another
-      # agument to the macro).  Using the standard quotes leaves a trailing
-      # single quote for some undetermined reason.
-    my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,changequote([,])[%s]changequote(`,'))dnl\n",
+    my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,%s)dnl\n",
       $name,
       $name_underscored,
       $cpp_type,
@@ -605,18 +495,10 @@ sub output_wrap_property($$$$$$)
     $self->append($str);
     $self->append("\n");
 
-    # If the property is not already read-only, and the property can be read, then add a second const accessor for a read-only propertyproxy:
-    if( ($proxy_suffix ne "_ReadOnly") && ($objProperty->get_readable()) )
+    # If the property is not already read-only, then add a second const accessor for a read-only propertyproxy:
+    if($proxy_suffix ne "_ReadOnly")
     {
-
-      # For the docs of the property (the final argument of the sprintf), the
-      # m4 quotes are changed, the new quotes are then used to quote the docs
-      # and then the quotes are changed back to the standard quotes.  This is
-      # done so that if there are commas in the docs, the contents after the
-      # docs are not lost (m4 thinks the contents after the comma is another
-      # agument to the macro).  Using the standard quotes leaves a trailing
-      # single quote for some undetermined reason.
-      my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,changequote([,])[%s]changequote(`,'))dnl\n",
+      my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,%s)dnl\n",
         $name,
         $name_underscored,
         $cpp_type,
@@ -625,6 +507,8 @@ sub output_wrap_property($$$$$$)
       );
       $self->append($str);
     }
+
+    
   }
 }
 
@@ -651,7 +535,7 @@ sub make_g2_from_g1($)
   my ($self) = @_;
 
   # Execute m4 to get *.g2 file:
-  system("$$self{m4path} $$self{m4args} \"$$self{tmpdir}/gtkmmproc_$$.g1\" > \"$$self{tmpdir}/gtkmmproc_$$.g2\"");
+  system("$$self{m4path} $$self{m4args} '$$self{tmpdir}/gtkmmproc_$$.g1' > '$$self{tmpdir}/gtkmmproc_$$.g2'");
   return ($? >> 8);
 }
 
@@ -667,7 +551,7 @@ sub write_sections_to_files()
 
   open(INPUT, '<', "$$self{tmpdir}/gtkmmproc_$$.g2"); # $$ is the process ID.
 
-  # open temporary file for each section
+  # open tempory file for each section
   open(OUTPUT_H,  '>', "$fname_h.tmp");
   open(OUTPUT_PH, '>', "$fname_ph.tmp");
   open(OUTPUT_CC, '>', "$fname_cc.tmp");
@@ -698,12 +582,7 @@ sub write_sections_to_files()
   foreach($fname_h, $fname_ph, $fname_cc)
   {
     # overwrite the source file only if it has actually changed
-
-    # Win32 does fail at this, so we do the two steps separately:
-    #system("cmp -s '$_.tmp' '$_' || cp '$_.tmp' '$_'" ; rm -f '$_.tmp');
-
-    system("cmp -s '$_.tmp' '$_' || cp '$_.tmp' '$_'");
-    system("rm -f '$_.tmp'");
+    system("cmp -s '$_.tmp' '$_' || cp '$_.tmp' '$_' ; rm -f '$_.tmp'");
   }
 }
 
@@ -743,7 +622,7 @@ sub convert_args_cpp_to_c($$$$;$)
        ${$c_param_types}[-1] eq "GError**" )
   {
     $num_cpp_args++;
-    $cpp_param_names = [@{$cpp_param_names},"gerror"];
+    $cpp_param_names = [@{$cpp_param_names},"error"];
     $cpp_param_types = [@{$cpp_param_types},"GError*&"];
   }
 
@@ -934,15 +813,5 @@ sub output_wrap_corba_method($$$$)
   $self->append($str);
 }
 
-sub output_implements_interface($$)
-{
-  my ($self, $interface, $ifdef) = @_;
-
-  my $str = sprintf("_IMPLEMENTS_INTERFACE_CC(%s, %s)dnl\n",
-  	$interface,
-  	$ifdef);
-
-  $self->append($str);
-}
 
 1; # indicate proper module load.
